#!/bin/sh
#
# instmon - INSTall MONitor - an installation monitoring tool
# Copyright (C) 1998 Vasilis Vasaitis (vvasaitis@waveland.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# initializes important variables with default values
init_data()
{
    package=""
    logdir="/usr/local/share/instmon"
    search="/"
    exclude="/proc /tmp /var /dev /home /usr/src /usr/local/src"
    fstype=""
    actions=""
}

# reads the system wide and user specific configuration files
read_conf_files()
{
    exclude_tmp="$exclude"
    [ -f /etc/instmon.conf ] && . /etc/instmon.conf
    exclude_tmp="$exclude_tmp $exclude"
    [ -f $HOME/.instmonrc ] && . $HOME/.instmonrc
    exclude="$exclude_tmp $exclude"
}

# gets the arguments from the command line
parse_cmd_line()
{
    while [ $# -gt 0 ]
    do
        case "$1" in
	    -i|--include)
		search="$search $2"
		shift
		shift
		;;
	    -x|--exclude)
		exclude="$exclude $2"
		shift
		shift
		;;
	    -s|--search)
	        search="$2"
		shift
		shift
		;;
	    -g|--logdir)
		logdir="$2"
		shift
		shift
		;;
	    --fstype)
		fstype="$fstype $2"
		shift
		shift
		;;
	    -l|--list)
		actions="$actions 2list"
		shift
		;;
	    --remove-files)
		actions="$actions 8remfiles"
		shift
		;;
	    --remove-log)
		actions="$actions 9remlog"
		shift
		;;
	    *)
		if [ -z "$package" ]
		then
		    package="$1"
		    shift
		else
		    printf "instmon: %s: only one package should be specified\n" "$1" 1>&2
		    exit 1
		fi
		;;
        esac
    done
    [ -z "$package" ] && package=$(basename $(pwd))
    printf "Monitoring package %s ...\n" "$package" 1>&2
}

# clean up $search/$exclude a bit
search_exclude_normalize()
{
    for var in search exclude
    do
	eval tmp=\"\$$var\"
	tmp=$(printf " %s " "$tmp" | tr -s " /" | sed "s/\/ / /g" | sed "s/  / \/ /g" | tr " " "\n" | sort -u | tr "\n" " ")
	eval $var=\"\$tmp\"
    done
}

# prepares the command line of find
setup_find_args()
{
    actionargs="-cnewer /tmp/instmon.$package -print"
    excludeargs=""
    if [ -n "$fstype" ]
    then
	for type in $fstype
	do
	    excludeargs="$excludeargs -not -fstype $type"
	done
	excludeargs="$excludeargs -prune -or"
    fi
    for path in $exclude
    do
	excludeargs="$excludeargs -path $path -prune -or"
    done
}

# records the current time (as the timestamp of a file)
record()
{
    printf "Recording timestamp ... " 1>&2
    : > /tmp/instmon.$package
    printf "done.\n" 1>&2
}

# detects the changes made to the file system since the recorded time
compare()
{
    printf "Detecting changes ... " 1>&2
    : > /tmp/instmon.files.$$
    for path in $search
    do
	find $path -path $path $actionargs -or $excludeargs $actionargs >> /tmp/instmon.files.$$ 2> /dev/null
    done
    printf "done.\n" 1>&2
}

# tidies /tmp and places the log in $logdir if possible
cleanup()
{
    rm -f /tmp/instmon.$package
    sort -u /tmp/instmon.files.$$ > /tmp/instlog.$package
    rm -f /tmp/instmon.files.$$
    mv -f /tmp/instlog.$package $logdir 2> /dev/null
}

perform_actions()
{
    while [ $# -gt 0 ]
    do
	case "$1" in
	    2list)
		printf "Listing log ... done.\n" 1>&2
		cat $logdir/instlog.$package
		;;
	    8remfiles)
		printf "Removing files ... " 1>&2
		rm -f $(tac $logdir/instlog.$package) 2> /dev/null
		rmdir $(tac $logdir/instlog.$package) 2> /dev/null
		printf "done.\n" 1>&2
		;;
	    9remlog)
		printf "Removing log ... " 1>&2
		rm -f $logdir/instlog.$package
		printf "done.\n" 1>&2
		;;
	esac
	shift
    done
}

# main program
cmdline="$*"
init_data
read_conf_files
parse_cmd_line $cmdline
if [ -n "$actions" ]
then
    if [ -f $logdir/instlog.$package ]
    then
	actions=$(printf "%s " "$actions" | tr " " "\n" | sort -u | tr "\n" " ")
	perform_actions $actions
    else
	printf "Log file not found.\n" 1>&2
	exit 1
    fi
elif [ -f /tmp/instmon.$package ]
then
    search_exclude_normalize
    setup_find_args
    compare
    cleanup
else
    record
fi

exit 0
